/*                                           -*- mode: c; c-basic-offset: 2 -*-
 *
 * This is kiba, a fun GNOME dock
 */

#ifndef __KIBA_DOCK_H__
#define __KIBA_DOCK_H__

#include "../config.h"
#undef HAVE_GLITZ

#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif
#ifdef HAVE_GLITZ
  #include <gdk/gdkx.h>
  #include <glitz-glx.h>
  #include <cairo-glitz.h>
#endif
#include <akamaru.h>

#define KIBA_TYPE_DOCK             (kiba_dock_get_type ())
#define KIBA_DOCK(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DOCK, KibaDock))
#define KIBA_DOCK_CLASS(obj)       (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_DOCK,  KibaDockClass))
#define KIBA_IS_DOCK(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_IS_DOCK_CLASS(obj)    (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_DOCK_GET_CLASS        (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DOCK, KibaDockClass))

#define KIBA_ERROR kiba_error_quark ()
#define gdkdisplay (Display*) gdk_x11_get_default_xdisplay ()

#define dsettings dock->settings
#define default_icon_size 40

GQuark kiba_error_quark (void);

typedef struct _KibaDock           KibaDock;
typedef struct _KibaDockClass      KibaDockClass;
typedef struct _KibaText           KibaText;
typedef struct _KibaSettings       KibaSettings;
typedef struct _KibaObject         KibaObject;
typedef struct _KibaPlugin         KibaPlugin;
typedef struct _PluginFunc         PluginFunc;
typedef struct _PluginHandle       PluginHandle;

/* plugin functions */
typedef void   (*init_t)    (KibaDock*, KibaPlugin*, void*, GError*);
typedef void   (*option_t)  (KibaDock*, char*, char*, char*);
typedef void   (*paint_t)   (KibaDock*, KibaObject*, cairo_t*);
typedef void   (*timeout_t) (KibaDock*, KibaObject*, GError*);
typedef void   (*menu_t)    (KibaDock*, KibaObject*, GtkMenu*);
typedef void   (*release_t) (KibaDock*, KibaObject*);
typedef void   (*scroll_t)  (KibaDock*, KibaObject*, int);
typedef void   (*free_t)    (KibaDock*);

typedef enum {
  KIBA_DOCK_POSITION_LEFT_EDGE_TOP,
  KIBA_DOCK_POSITION_LEFT_EDGE_CENTER,
  KIBA_DOCK_POSITION_LEFT_EDGE_BOTTOM,

  KIBA_DOCK_POSITION_RIGHT_EDGE_TOP,
  KIBA_DOCK_POSITION_RIGHT_EDGE_CENTER,
  KIBA_DOCK_POSITION_RIGHT_EDGE_BOTTOM,

  KIBA_DOCK_POSITION_TOP_EDGE_LEFT,
  KIBA_DOCK_POSITION_TOP_EDGE_CENTER,
  KIBA_DOCK_POSITION_TOP_EDGE_RIGHT,

  KIBA_DOCK_POSITION_BOTTOM_EDGE_LEFT,
  KIBA_DOCK_POSITION_BOTTOM_EDGE_CENTER,
  KIBA_DOCK_POSITION_BOTTOM_EDGE_RIGHT
} KibaDockPosition;

typedef struct {
  const char     *name;
  GtkOrientation  orientation;
  int             halignment;
  int             valignment;
  GdkPoint        anchor;
} KibaDockLayoutData;

static const KibaDockLayoutData kiba_layout_data[] = {
  { "left-edge-top",      GTK_ORIENTATION_VERTICAL,   0, 0, { 0, 1 } },
  { "left-edge-center",   GTK_ORIENTATION_VERTICAL,   0, 1, { 0, 2 } },
  { "left-edge-bottom",   GTK_ORIENTATION_VERTICAL,   0, 2, { 0, 3 } },
  { "right-edge-top",     GTK_ORIENTATION_VERTICAL,   2, 0, { 4, 1 } },
  { "right-edge-center",  GTK_ORIENTATION_VERTICAL,   2, 1, { 4, 2 } },
  { "right-edge-bottom",  GTK_ORIENTATION_VERTICAL,   2, 2, { 4, 3 } },
  { "top-edge-left",      GTK_ORIENTATION_HORIZONTAL, 0, 0, { 1, 0 } },
  { "top-edge-center",    GTK_ORIENTATION_HORIZONTAL, 1, 0, { 2, 0 } },
  { "top-edge-right",     GTK_ORIENTATION_HORIZONTAL, 2, 0, { 3, 0 } },
  { "bottom-edge-left",   GTK_ORIENTATION_HORIZONTAL, 0, 2, { 1, 4 } },
  { "bottom-edge-center", GTK_ORIENTATION_HORIZONTAL, 1, 2, { 2, 4 } },
  { "bottom-edge-right",  GTK_ORIENTATION_HORIZONTAL, 2, 2, { 3, 4 } }
};

struct _KibaPlugin {
  char       *name;
  void       *plugin_struct;
  void       *option_struct;
};

struct _PluginFunc
{
  char       *plugin_name;
  init_t      init_func;
  option_t    option_changed_func;
  paint_t     paint_func;
  timeout_t   timeout_func;
  menu_t      menu_func;
  release_t   release_func;
  scroll_t    scroll_func;
  free_t      free_func;
};

struct _KibaDock
{
  GtkWidget           widget;
  KibaText           *text;
  KibaSettings       *settings;

  /* launcher related variables */
  GList              *objects;             //List of all kiba-objects,                 content: KibaObject's
  gint                num_objects;         //number of objects
  GList              *plugins;             //List of all running Plugins,              content: KibaPlugins's
  GList              *plugin_functions;    //List contains functions from the plugins, content: PluginFunc's
  GList              *plugin_handles;      //saves the opened librarys

  KibaObject         *dragging_object;
  KibaObject         *prelight_object;
  KibaObject         *exec_object;
  KibaObject         *last_text_object;

  /* physic engine related variables */
  AkamaruModel        model;
  AkamaruAnchor      *anchor;              //anchor for object->spring
  AkamaruAnchor      *mouse_anchor;
  AkamaruAnchor      *last_rope_anchor;
  AkamaruAnchor      *first_rope_anchor;

  /* rendering related variables */
  GdkPixmap          *pixmap;
  GdkGC              *gc;
  gboolean            exec_pulse;
  gboolean            rerender;
  gboolean            zoom_exec;
  double              zoom_exec_size;

  /* options */
  char               *settings_file; /* this stores the settings file content. */

  /* geometry variables */
  KibaDockPosition    position;
  KibaDockLayoutData *d;
  GdkRectangle        geometry;           //monitor/screen geometry
  GtkAllocation       allocation;         //dock-size
  GtkAllocation       last_allocation;    //dock-size before allocation
  gboolean            animate_dock;
  gboolean            animated_placement;
  gboolean            changing_position;
  gboolean            dragging_dock;
  gboolean            hiding;
  gboolean            unhiding;
  gint                drag_distance;
  gint                drag_offset_x;
  gint                drag_offset_y;
  gint                dx, dy;
  gint                hide_requests;

  /* animated bg */
  GdkRectangle        last_mouse_position;
  double              bg_offset;
  double              old_bg_offset;

  /* drag animation */
  GdkWindow          *drag_window;
  GdkPixmap          *drag_pixmap;
  GdkGC              *drag_gc;
  GdkRectangle        drag_geometry;
  gboolean            drag_motion;
  double              drag_window_offset;

  /* parameter */
  char               *cwd;               //current working directory
  gboolean            benchmark;
  gint                redraws;
  gint                events;
};

struct _KibaDockClass
{
  GtkWidgetClass parent_class;
};

struct _KibaText
{
  GdkWindow       *window;
  GdkPixmap       *pixmap;
  GdkGC           *gc;
  GdkRectangle     animation_stepsize;
  GdkRectangle     geometry;
  GdkRectangle     geometry_animate;
  GdkRectangle     geometry_size;
  gboolean         animate;
  gboolean         hidden;
  gboolean         showing;
  gint             hide_startpoint;
  gint             requests;
  char            *rendered_text; /* saves the last rendered text */
  double           last_size;
};

typedef struct {
  KibaDock      *dock;
  AkamaruObject *object;
  double         spacer_size;
} AddSpacerClosure;

void
kiba_reload                (KibaDock *dock);
void
kiba_get_rgb_from_string   (const char *html_color, double *r, double *g, double *b);
void
kiba_create_menu           (KibaDock *dock, KibaObject *object, GdkEventButton *event);
GdkWindow*
kiba_create_drag_window    (KibaDock *dock);
KibaPlugin*
kiba_get_plugin            (KibaDock *dock, gchar *plugin_name);
int
copyfile                   (char *fromfile, char *tofile);

#endif
