
#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#define PIXMAP_PATH "/usr/share/pixmaps"

#define ZOOM_AREA (dsettings->zoom_area_factor*object->size)
// #define ZOOM_AREA (1.5*object->normal_size*dsettings->zoom_factor)
#define MAX_SIZE (int) object->normal_size*dsettings->zoom_factor
#define PULSE_AREA (1.5 * lgconf->icon_size * lgconf->pulse_zoom_factor)
#define MAX_PULSE_SIZE (dock->exec_pulse == TRUE ? (object->normal_size * dsettings->pulse_exec_zoom_factor) : (object->normal_size*dsettings->pulse_zoom_factor))
#define ROTATE_AREA (dsettings->rotate_area_factor*object->normal_size)
#define ROPE_OBJECT_DISTANCE (dsettings->rope_object_distance / 10)

#define DO_FOR_ALL_OBJECTS(a); \
  GList *g_list; \
  for (g_list = dock->objects; g_list != NULL; g_list = g_list->next) \
  { \
    KibaObject *object = g_list->data; \
    a; \
  }

#define DO_FOR_ALL_PLUGINS(a); \
  GList *g_list; \
  for (g_list = dock->plugins; g_list != NULL; g_list = g_list->next) \
  { \
    KibaPlugin *plugin = g_list->data; \
    a; \
  }

#ifndef __KIBA_DOCK_H__
  typedef struct _KibaObject         KibaObject;
#endif

/* this struct saves a kiba objects, like a launcher, clock... */
struct _KibaObject {
  KibaPlugin               *plugin;
  gint                      plugin_num;

  GdkWindow                *window;
  gboolean                  at_dock;
  gboolean                  dragging;
  gboolean                  exec;
  gboolean                  prelight;   /* true when object is prelighted */
  char                     *name;       /* text for pango rendering */
  char                     *group;

  /* physic variables */
  AkamaruObject            *object;
  AkamaruSpacer            *spacer;
  AkamaruSpring            *spring;
  AkamaruSpring            *rope_spring;
  AkamaruString            *rope_string;
  GList                    *wobbly_spring;
  double                    velocity_x;
  double                    velocity_y;

  /* rendering related variables */
  GdkPixbuf                *bg_pixbuf;
  GdkPixbuf                *fg_pixbuf;
  GdkPixmap                *pixmap;
  GdkGC                    *gc;
  cairo_surface_t          *background_surface;
  cairo_surface_t          *foreground_surface;
  PangoLayout              *pango_layout;       //saves the pango layout

/*
#ifdef HAVE_GLITZ
  glitz_drawable_format_t  *glitz_draw_format;
  glitz_drawable_t         *glitz_drawable;
  glitz_format_t           *glitz_format;
#endif
*/

#ifdef HAVE_SVG
  RsvgHandle               *bg_svg_handle;
  RsvgHandle               *fg_svg_handle;
#endif

  gboolean                  rerender;
  gboolean                  rerender_pulse;
  gboolean                  rerender_text;
  gboolean                  bgsurface_needs_update;
  gboolean                  fgsurface_needs_update;

  /* pulse effect */
  GdkWindow                *pulse_window;
  GdkPixmap                *pulse_pixmap;
  GdkGC                    *pulse_gc;
  gboolean                  pulse_exec;
  double                    pulse_size;  /* size of the pulse window */
  gint                      pulse_count;

  /* geometry */
  GdkRectangle              win_position;
  GdkRectangle              previous_position;
  double                    drawn_rotate_angle;
  double                    rotate_angle;
  double                    scale_factor; /* scale factor for rotation */
  double                    normal_size; /* gconf size value */
  double                    size;        /* object size */
  double                    buffer_size;
//TODO replace size with height and width
  double                    buffer_width;
  double                    buffer_height;
  double                    width;
  double                    height;
};

void
kiba_object_create_window          (KibaDock *dock, KibaObject *object);
GdkWindow *
kiba_object_create_pulse_window    (KibaObject *object);
KibaObject*
kiba_get_object_for_window         (KibaDock  *dock, GdkWindow *window);
KibaObject*
kiba_get_object_under_pointer      (KibaDock  *dock);
void
kiba_object_timeout                (KibaDock *dock, KibaObject *object, gboolean object_size_modified, gboolean objects_at_dock, gboolean fast_motion);
void
kiba_object_free                   (KibaDock *dock, KibaObject *object);

void
kiba_object_paint                  (KibaDock *dock, KibaObject *object, PluginFunc *plugin_func);
void
kiba_pulse_paint                   (KibaDock *dock, KibaObject *object);
gboolean
kiba_text_paint                    (KibaDock *dock, KibaObject *object, double fade_factor);
void
fill_surface_buffer                (cairo_t* pSourceContext, KibaObject *object, gboolean background);
void
kiba_update_surface_buffer         (KibaObject *object, gboolean background);
gboolean
kiba_load_png                      (KibaDock *dock, gchar *icon, int size, GdkPixbuf **pixbuf, GError **error);
#ifdef HAVE_SVG
gboolean
kiba_load_svg                      (KibaObject *object, char* path, RsvgHandle **handle, GError **error);
#endif
void
kiba_text_update_pango_for_object  (KibaObject *object, KibaSettings *settings, KibaText *text);
void
kiba_rotate_object                 (KibaDock *dock, KibaObject *object, cairo_t *pixmap_context);
void
kiba_zoom_calculate_size           (KibaDock *dock, KibaObject *object, int mouse_l_x, int mouse_l_y, gboolean object_size_modified);

void
kiba_hide_objects                  (KibaDock *dock);
void
kiba_show_objects                  (KibaDock *dock);
static void
kiba_change_object_win_position    (KibaDock *dock, KibaObject *object, gint mouse_x, gint mouse_y);
void
kiba_object_update_position        (KibaDock *dock, KibaObject *object, gboolean reload);
void
kiba_reorder_objectlist            (KibaDock *dock);
void
kiba_update_object_positions       (KibaDock *dock);
/*
void
kiba_object_position_changed       (KibaDock *dock, KibaObject *object, GConfEntry *entry);
*/
void
kiba_update_rope_positions         (KibaDock *dock, KibaObject *object);
gboolean
kiba_taskbar_shows_app (KibaDock *dock, GList *app_list);

void
kiba_object_set_options            (KibaDock *dock, void *option_struct, GconfOption gconf_options[], int array_elements);
gboolean
kiba_object_exec                   (gpointer data);

/* GdkPixbuf RGBA C-Source image dump 1-byte-run-length-encoded */

#ifdef __SUNPRO_C
#pragma align 4 (fallback_icon_data)
#endif
#ifdef __GNUC__
static const guint8 fallback_icon_data[] __attribute__ ((__aligned__ (4))) = 
#else
static const guint8 fallback_icon_data[] = 
#endif
{ ""
  /* Pixbuf magic (0x47646b50) */
  "GdkP"
  /* length: header (24) + pixel_data (8325) */
  "\0\0\40\235"
  /* pixdata_type (0x2010002) */
  "\2\1\0\2"
  /* rowstride (256) */
  "\0\0\1\0"
  /* width (64) */
  "\0\0\0@"
  /* height (64) */
  "\0\0\0@"
  /* pixel_data: */
  "\377\0\0\0\0\240\0\0\0\0\202\220\3\3\1\267\0\0\0\0\7\220\3\3\4\215\3"
  "\3:\215\3\3v\215\3\3\236\215\3\3\304\215\3\3\343\212\3\3\360\202\206"
  "\3\3\374\7\212\3\3\360\215\3\3\343\215\3\3\304\215\3\3\236\215\3\3v\215"
  "\3\3:\220\3\3\4\255\0\0\0\0\11\220\3\3\7\215\3\3S\212\3\3\256\206\2\2"
  "\362\201\0\0\377\177\0\0\377~\0\0\377|\0\0\377{\0\0\377\204z\0\0\377"
  "\11{\0\0\377|\0\0\377~\0\0\377\177\0\0\377\201\0\0\377\206\2\2\362\212"
  "\3\3\256\215\3\3S\220\3\3\7\250\0\0\0\0\32\220\3\3\24\212\3\3\203\206"
  "\2\2\355\201\0\0\377~\0\0\377{\0\0\377\217\25\25\377\260HH\377\312qq"
  "\377\337\231\231\377\363\277\277\377\373\324\324\377\377\347\347\377"
  "\377\351\351\377\374\327\327\377\363\303\303\377\341\234\234\377\312"
  "tt\377\260JJ\377\220\25\25\377{\0\0\377~\0\0\377\201\0\0\377\206\2\2"
  "\355\212\3\3\203\220\3\3\24\244\0\0\0\0\36\220\3\3\14\212\3\3\210\205"
  "\2\2\365\200\0\0\377|\0\0\377\224\27\27\377\303WW\377\360\231\231\377"
  "\374\261\261\377\374\267\267\377\373\274\274\377\372\302\302\377\372"
  "\311\311\377\373\317\317\377\372\324\324\377\372\325\325\377\373\323"
  "\323\377\373\314\314\377\372\306\306\377\373\301\301\377\374\272\272"
  "\377\374\266\266\377\360\234\234\377\303YY\377\224\30\30\377|\0\0\377"
  "\200\0\0\377\205\2\2\365\212\3\3\210\220\3\3\14\241\0\0\0\0\40\215\3"
  "\3P\206\2\2\342\200\0\0\377|\0\0\377\241$$\377\336pp\377\371\227\227"
  "\377\366\231\231\377\365\240\240\377\365\250\250\377\365\262\262\377"
  "\366\272\272\377\367\305\305\377\370\312\312\377\370\276\276\377\370"
  "\255\255\377\370\256\256\377\370\300\300\377\370\314\314\377\367\310"
  "\310\377\366\277\277\377\366\265\265\377\364\254\254\377\365\243\243"
  "\377\366\236\236\377\371\232\232\377\336rr\377\241%%\377|\0\0\377\200"
  "\0\0\377\206\2\2\342\215\3\3P\236\0\0\0\0\17\220\3\3\2\212\3\3\212\205"
  "\1\1\377~\0\0\377\226\25\25\377\335dd\377\366\205\205\377\363\210\210"
  "\377\362\220\220\377\362\231\231\377\362\235\235\377\362~~\377\362ZZ"
  "\377\36222\377\362\34\34\377\202\362\16\16\377\202\362\17\17\377\21\362"
  "\16\16\377\362\15\15\377\362\35\35\377\36233\377\363]]\377\363\203\203"
  "\377\362\241\241\377\362\235\235\377\362\222\222\377\363\214\214\377"
  "\366\210\210\377\335hh\377\226\26\26\377~\0\0\377\205\1\1\377\212\3\3"
  "\212\220\3\3\2\233\0\0\0\0\20\215\3\3\16\206\3\3\266\201\1\1\377\177"
  "\0\0\377\302@@\377\362uu\377\361zz\377\356\201\201\377\357\204\204\377"
  "\357__\377\36011\377\360\23\23\377\360\22\22\377\360\24\24\377\360\26"
  "\26\377\360\30\30\377\204\360\31\31\377\22\357\31\31\377\357\30\30\377"
  "\357\27\27\377\357\25\25\377\356\23\23\377\356\20\20\377\356\23\23\377"
  "\35755\377\356ee\377\357\211\211\377\357\203\203\377\360}}\377\362xx"
  "\377\302@@\377\177\0\0\377\201\1\1\377\206\3\3\266\215\3\3\16\231\0\0"
  "\0\0\15\215\3\3$\206\2\2\326\200\0\0\377\205\5\5\377\325NN\377\360mm"
  "\377\355rr\377\354dd\377\355==\377\356\26\26\377\356\23\23\377\356\25"
  "\25\377\356\27\27\377\203\355\30\30\377\213\354\30\30\377\15\354\27\27"
  "\377\354\25\25\377\354\22\22\377\354\31\31\377\354DD\377\354ll\377\355"
  "tt\377\356oo\377\325PP\377\205\5\5\377\200\0\0\377\206\2\2\326\215\3"
  "\3$\227\0\0\0\0\13\215\3\3$\206\2\2\353\177\0\0\377\217\15\15\377\341"
  "TT\377\355ee\377\353WW\377\353++\377\353\22\22\377\353\25\25\377\353"
  "\27\27\377\204\352\30\30\377\217\352\27\27\377\14\352\26\26\377\352\25"
  "\25\377\352\23\23\377\351\22\22\377\35144\377\353``\377\355gg\377\340"
  "VV\377\217\16\16\377\177\0\0\377\206\2\2\353\215\3\3$\225\0\0\0\0\11"
  "\215\3\3\16\206\2\2\326\177\0\0\377\233\27\27\377\350UU\377\353PP\377"
  "\350((\377\350\22\22\377\350\25\25\377\206\350\26\26\377\202\350\25\25"
  "\377\202\350\24\24\377\1\350\25\25\377\216\350\26\26\377\12\350\25\25"
  "\377\350\23\23\377\347\22\22\377\34711\377\352XX\377\347VV\377\233\26"
  "\26\377\177\0\0\377\206\2\2\326\215\3\3\16\223\0\0\0\0\11\220\3\3\2\206"
  "\3\3\266\200\1\1\377\220\15\15\377\346LL\377\35011\377\346\23\23\377"
  "\346\24\24\377\346\25\25\377\206\346\26\26\377\10\345\23\23\377\345\17"
  "\17\377\345\20\20\377\35100\377\347\37\37\377\345\15\15\377\345\21\21"
  "\377\346\25\25\377\213\346\26\26\377\203\345\26\26\377\11\345\25\25\377"
  "\344\24\24\377\344\25\25\377\346==\377\345QQ\377\220\15\15\377\200\1"
  "\1\377\206\3\3\266\220\3\3\2\222\0\0\0\0\5\212\3\3\212\201\2\2\377\206"
  "\7\7\377\33411\377\347\32\32\377\202\344\24\24\377\207\344\25\25\377"
  "\12\344\21\21\377\343\23\23\377\356xx\377\370\301\301\377\370\302\302"
  "\377\371\303\303\377\366\250\250\377\35088\377\344\17\17\377\344\24\24"
  "\377\204\344\25\25\377\2\343\24\24\377\343\22\22\377\202\342\20\20\377"
  "\2\342\22\22\377\342\24\24\377\205\342\25\25\377\7\342\24\24\377\342"
  "\22\22\377\344\"\"\377\333<<\377\205\6\6\377\201\2\2\377\212\3\3\212"
  "\221\0\0\0\0\6\215\3\3P\205\3\3\377\177\3\3\377\317\26\26\377\345\23"
  "\23\377\342\23\23\377\210\342\24\24\377\13\342\22\22\377\341\15\15\377"
  "\361\227\227\377\370\276\276\377\361\270\270\377\345\264\264\377\341"
  "\263\263\377\343\270\270\377\351\276\276\377\34488\377\341\20\20\377"
  "\202\341\24\24\377\12\340\23\23\377\337\17\17\377\340\16\16\377\3442"
  "2\377\352^^\377\352[[\377\344//\377\340\14\14\377\340\17\17\377\340\23"
  "\23\377\205\340\24\24\377\6\341\23\23\377\343\25\25\377\315\37\37\377"
  "\200\2\2\377\205\2\2\377\215\3\3P\217\0\0\0\0\6\220\3\3\14\206\3\3\342"
  "\177\2\2\377\271\14\14\377\344\25\25\377\341\24\24\377\211\340\24\24"
  "\377\4\340\20\20\377\350OO\377\367\300\300\377\353\266\266\377\202\337"
  "\263\263\377\202\340\263\263\377\21\340\266\266\377\332\246\246\377\323"
  "\11\11\377\337\24\24\377\336\23\23\377\335\15\15\377\34333\377\365\273"
  "\273\377\372\334\334\377\372\332\332\377\370\331\331\377\371\334\334"
  "\377\365\262\262\377\341**\377\335\16\16\377\336\23\23\377\336\24\24"
  "\377\204\336\23\23\377\6\337\23\23\377\342\23\23\377\270\15\15\377\177"
  "\2\2\377\206\3\3\342\220\3\3\14\216\0\0\0\0\5\212\3\3\210\201\2\2\377"
  "\222\5\5\377\343\24\24\377\337\23\23\377\205\336\23\23\377\203\335\23"
  "\23\377\202\334\23\23\377\4\333\14\14\377\355\177\177\377\361\274\274"
  "\377\337\263\263\377\204\340\263\263\377\20\340\264\264\377\343\277\277"
  "\377\252\25\25\377\337\22\22\377\334\16\16\377\34288\377\371\331\331"
  "\377\370\324\324\377\357\317\317\377\347\314\314\377\346\314\314\377"
  "\346\315\315\377\350\321\321\377\357\325\325\377\336,,\377\334\15\15"
  "\377\206\334\22\22\377\5\335\22\22\377\341\23\23\377\222\5\5\377\201"
  "\2\2\377\212\3\3\210\215\0\0\0\0\5\220\3\3\24\206\3\3\365~\2\2\377\315"
  "\20\20\377\337\22\22\377\203\333\22\22\377\210\332\22\22\377\4\331\12"
  "\12\377\356\221\221\377\352\271\271\377\337\262\262\377\205\340\262\262"
  "\377\10\345\275\275\377\241%%\377\323\15\15\377\331\17\17\377\365\277"
  "\277\377\367\324\324\377\351\315\315\377\346\314\314\377\204\347\314"
  "\314\377\5\347\321\321\377\334\260\260\377\321\12\12\377\333\21\21\377"
  "\332\22\22\377\203\331\22\22\377\202\330\22\22\377\5\332\23\23\377\312"
  "\17\17\377~\2\2\377\206\3\3\365\220\3\3\24\214\0\0\0\0\4\212\3\3\203"
  "\201\2\2\377\231\6\6\377\335\23\23\377\205\330\22\22\377\13\327\17\17"
  "\377\327\14\14\377\327\12\12\377\327\13\13\377\327\16\16\377\330\20\20"
  "\377\330\22\22\377\327\14\14\377\347mm\377\345\272\272\377\336\262\262"
  "\377\204\337\262\262\377\7\340\264\264\377\343\276\276\377\223\1\1\377"
  "\305\11\11\377\341DD\377\372\332\332\377\355\316\316\377\206\347\314"
  "\314\377\4\350\315\315\377\353\331\331\377\254))\377\330\16\16\377\206"
  "\326\21\21\377\5\327\21\21\377\333\22\22\377\231\6\6\377\201\2\2\377"
  "\212\3\3\203\213\0\0\0\0\5\220\3\3\7\206\3\3\355~\2\2\377\310\17\17\377"
  "\330\21\21\377\203\326\21\21\377\10\326\20\20\377\325\14\14\377\331\34"
  "\34\377\346hh\377\353\210\210\377\351{{\377\341NN\377\325\13\13\377\202"
  "\326\16\16\377\4\332$$\377\350\271\271\377\337\265\265\377\337\263\263"
  "\377\202\337\262\262\377\10\337\263\263\377\342\271\271\377\305ss\377"
  "\221\0\0\377\316\11\11\377\347qq\377\366\326\326\377\346\314\314\377"
  "\207\347\314\314\377\4\353\326\326\377\251TT\377\303\11\11\377\325\20"
  "\20\377\206\324\20\20\377\5\326\21\21\377\306\16\16\377~\2\2\377\206"
  "\3\3\355\220\3\3\7\212\0\0\0\0\5\215\3\3S\201\2\2\377\217\4\4\377\331"
  "\21\21\377\325\20\20\377\203\324\20\20\377\30\323\14\14\377\331//\377"
  "\356\242\242\377\357\245\245\377\350\241\241\377\342\241\241\377\346"
  "\245\245\377\345\203\203\377\324\14\14\377\324\16\16\377\323\12\12\377"
  "\332MM\377\337\270\270\377\340\271\271\377\340\267\267\377\340\270\270"
  "\377\342\274\274\377\322\226\226\377\225\7\7\377\233\5\5\377\324\12\12"
  "\377\346xx\377\362\326\326\377\346\314\314\377\207\347\314\314\377\4"
  "\354\326\326\377\257[[\377\260\4\4\377\324\21\21\377\207\322\20\20\377"
  "\4\327\21\21\377\216\4\4\377\201\2\2\377\215\3\3S\212\0\0\0\0\4\212\3"
  "\3\256\200\2\2\377\260\12\12\377\325\21\21\377\203\322\20\20\377\5\322"
  "\17\17\377\320\14\14\377\353\227\227\377\355\241\241\377\334\234\234"
  "\377\202\327\233\233\377\22\327\234\234\377\330\242\242\377\315SS\377"
  "\322\13\13\377\322\16\16\377\321\11\11\377\266\37\37\377\274hh\377\316"
  "\217\217\377\307{{\377\261HH\377\220\0\0\377\222\2\2\377\306\15\15\377"
  "\321\13\13\377\333JJ\377\360\327\327\377\345\314\314\377\207\346\314"
  "\314\377\4\354\330\330\377\230,,\377\243\4\4\377\323\20\20\377\207\320"
  "\17\17\377\4\323\17\17\377\257\12\12\377\200\2\2\377\212\3\3\256\211"
  "\0\0\0\0\5\220\3\3\4\206\3\3\362~\2\2\377\315\17\17\377\321\17\17\377"
  "\203\320\17\17\377\5\320\15\15\377\325))\377\357\251\251\377\335\234"
  "\234\377\326\233\233\377\203\327\233\233\377\21\330\236\236\377\313\203"
  "\203\377\270\3\3\377\317\16\16\377\316\15\15\377\317\13\13\377\266\5"
  "\5\377\227\0\0\377\215\0\0\377\222\1\1\377\243\5\5\377\311\15\15\377"
  "\320\16\16\377\316\14\14\377\316\22\22\377\360\313\313\377\346\317\317"
  "\377\206\346\314\314\377\6\350\320\320\377\340\300\300\377\177\0\0\377"
  "\260\5\5\377\316\14\14\377\315\17\17\377\202\316\17\17\377\1\316\16\16"
  "\377\204\315\16\16\377\4\312\16\16\377~\2\2\377\206\3\3\362\220\3\3\4"
  "\210\0\0\0\0\5\215\3\3:\202\3\3\377\213\3\3\377\323\17\17\377\316\16"
  "\16\377\203\315\16\16\377\4\313\13\13\377\32599\377\354\247\247\377\326"
  "\233\233\377\204\327\233\233\377\22\330\235\235\377\323\225\225\377\237"
  "\0\0\377\317\16\16\377\314\16\16\377\315\16\16\377\316\16\16\377\317"
  "\16\16\377\316\15\15\377\320\17\17\377\317\16\16\377\315\16\16\377\314"
  "\16\16\377\314\15\15\377\313\7\7\377\327KK\377\350\330\330\377\347\317"
  "\317\377\204\346\314\314\377\13\350\320\320\377\355\330\330\377\2320"
  "0\377}\0\0\377\277\0\0\377\307\0\0\377\306\0\0\377\306\1\1\377\307\5"
  "\5\377\311\12\12\377\312\16\16\377\203\312\15\15\377\4\317\16\16\377"
  "\212\3\3\377\202\3\3\377\215\3\3:\210\0\0\0\0\4\215\3\3v\201\2\2\377"
  "\237\7\7\377\315\17\17\377\202\312\16\16\377\202\312\15\15\377\4\312"
  "\13\13\377\315\31\31\377\350\250\250\377\325\234\234\377\204\326\233"
  "\233\377\4\330\237\237\377\307vv\377\225\0\0\377\315\16\16\377\212\312"
  "\15\15\377\5\312\13\13\377\311\5\5\377\305LL\377\343\313\313\377\353"
  "\327\327\377\202\352\324\324\377\6\353\327\327\377\342\305\305\377\235"
  "::\377|\0\0\377\241\0\0\377\307\0\0\377\205\304\0\0\377\10\304\1\1\377"
  "\306\7\7\377\310\15\15\377\310\14\14\377\313\15\15\377\236\7\7\377\201"
  "\2\2\377\215\3\3v\210\0\0\0\0\4\215\3\3\236\200\2\2\377\255\10\10\377"
  "\312\14\14\377\205\310\14\14\377\4\307\5\5\377\335pp\377\326\241\241"
  "\377\326\234\234\377\202\326\233\233\377\5\327\235\235\377\332\244\244"
  "\377\244$$\377\235\2\2\377\313\14\14\377\211\310\14\14\377\202\307\14"
  "\14\377\12\307\13\13\377\304\2\2\377\235\7\7\377\241FF\377\271vv\377"
  "\270qq\377\242BB\377\202\2\2\377{\0\0\377\226\0\0\377\202\300\0\0\377"
  "\3\277\0\0\377\300\0\0\377\301\0\0\377\205\302\0\0\377\5\304\7\7\377"
  "\307\15\15\377\253\11\11\377\200\2\2\377\215\3\3\236\210\0\0\0\0\4\215"
  "\3\3\304\177\2\2\377\267\12\12\377\307\14\14\377\205\306\14\14\377\13"
  "\305\12\12\377\310\17\17\377\312pp\377\330\245\245\377\330\243\243\377"
  "\330\244\244\377\326\235\235\377\255;;\377\223\0\0\377\257\7\7\377\310"
  "\15\15\377\203\306\14\14\377\204\305\14\14\377\202\304\14\14\377\202"
  "\304\15\15\377\5\302\6\6\377\276\0\0\377\277\0\0\377\255\0\0\377\223"
  "\0\0\377\202\205\0\0\377\12\223\0\0\377\262\20\20\377\335mm\377\352\250"
  "\250\377\356\275\275\377\351\251\251\377\333nn\377\303\21\21\377\276"
  "\0\0\377\277\0\0\377\204\300\0\0\377\4\301\2\2\377\265\10\10\377\177"
  "\3\3\377\215\3\3\304\210\0\0\0\0\4\215\3\3\343\177\2\2\377\300\12\12"
  "\377\305\13\13\377\206\304\13\13\377\13\304\11\11\377\274\3\3\377\243"
  "\33\33\377\254<<\377\247//\377\227\10\10\377\222\0\0\377\242\4\4\377"
  "\304\14\14\377\304\13\13\377\303\13\13\377\210\302\13\13\377\2\302\14"
  "\14\377\300\6\6\377\203\274\0\0\377\5\275\0\0\377\277\0\0\377\276\0\0"
  "\377\275\0\0\377\336ww\377\202\373\361\361\377\10\371\356\356\377\367"
  "\354\354\377\366\355\355\377\371\361\361\377\372\361\361\377\335zz\377"
  "\273\0\0\377\275\0\0\377\203\276\0\0\377\4\277\0\0\377\271\0\0\377\177"
  "\3\3\377\215\3\3\343\210\0\0\0\0\2\212\3\3\360~\2\2\377\211\302\12\12"
  "\377\10\302\11\11\377\300\10\10\377\255\4\4\377\241\3\3\377\244\4\4\377"
  "\263\10\10\377\302\12\12\377\301\12\12\377\211\300\12\12\377\3\300\13"
  "\13\377\276\6\6\377\271\0\0\377\204\272\0\0\377\6\271\0\0\377\266\0\0"
  "\377\344\231\231\377\373\362\362\377\370\352\352\377\364\347\347\377"
  "\202\361\347\347\377\202\362\347\347\377\4\362\352\352\377\365\362\362"
  "\377\342\235\235\377\271\0\0\377\205\274\0\0\377\2\177\3\3\377\212\3"
  "\3\360\207\0\0\0\0\4\220\3\3\1\206\3\3\374~\2\2\377\303\12\12\377\202"
  "\300\12\12\377\204\277\12\12\377\204\276\12\12\377\1\277\12\12\377\202"
  "\300\12\12\377\1\277\12\12\377\212\276\12\12\377\2\276\11\11\377\273"
  "\5\5\377\206\270\0\0\377\4\266\0\0\377\321\\\\\377\373\363\363\377\366"
  "\351\351\377\202\361\347\347\377\205\362\347\347\377\4\362\351\351\377"
  "\365\363\363\377\307``\377\271\0\0\377\203\272\0\0\377\4\275\0\0\377"
  "\177\3\3\377\206\3\3\374\220\3\3\1\206\0\0\0\0\4\220\3\3\1\206\3\3\374"
  "\177\2\2\377\300\11\11\377\204\274\11\11\377\6\273\7\7\377\272\3\3\377"
  "\274\11\11\377\274\6\6\377\273\4\4\377\273\7\7\377\207\274\11\11\377"
  "\207\274\10\10\377\1\271\5\5\377\206\266\0\0\377\5\265\0\0\377\264\0"
  "\0\377\366\340\340\377\370\353\353\377\361\347\347\377\210\362\347\347"
  "\377\4\363\353\353\377\354\344\344\377\241\0\0\377\271\0\0\377\202\270"
  "\0\0\377\4\273\0\0\377\177\3\3\377\206\3\3\374\220\3\3\1\207\0\0\0\0"
  "\3\212\3\3\360\177\2\2\377\273\10\10\377\203\272\10\10\377\10\271\6\6"
  "\377\273\14\14\377\324bb\377\335\207\207\377\335\205\205\377\315MM\377"
  "\271\2\2\377\272\7\7\377\213\272\10\10\377\2\271\10\10\377\266\3\3\377"
  "\207\264\0\0\377\5\263\0\0\377\30322\377\374\365\365\377\363\350\350"
  "\377\361\347\347\377\210\362\347\347\377\4\362\350\350\377\370\365\365"
  "\377\23266\377\265\0\0\377\202\266\0\0\377\3\265\0\0\377\177\3\3\377"
  "\212\3\3\360\210\0\0\0\0\4\215\3\3\343\177\2\2\377\265\7\7\377\271\10"
  "\10\377\202\270\10\10\377\10\267\3\3\377\315TT\377\333\201\201\377\322"
  "}}\377\313||\377\315\202\202\377\27655\377\270\4\4\377\207\270\7\7\377"
  "\203\267\7\7\377\2\266\6\6\377\263\2\2\377\210\262\0\0\377\4\260\0\0"
  "\377\317^^\377\371\362\362\377\361\347\347\377\212\362\347\347\377\3"
  "\367\362\362\377\252cc\377\243\0\0\377\202\265\0\0\377\3\261\0\0\377"
  "\200\3\3\377\215\3\3\343\210\0\0\0\0\4\215\3\3\304\200\3\3\377\253\6"
  "\6\377\267\7\7\377\202\266\7\7\377\3\264\0\0\377\327ww\377\322}}\377"
  "\202\307zz\377\4\311~~\377\266VV\377\253\0\0\377\267\6\6\377\204\265"
  "\6\6\377\204\264\6\6\377\1\262\4\4\377\212\260\0\0\377\3\256\0\0\377"
  "\320gg\377\370\362\362\377\213\362\347\347\377\10\367\362\362\377\257"
  "ll\377\223\0\0\377\264\0\0\377\263\0\0\377\250\0\0\377\200\3\3\377\215"
  "\3\3\304\210\0\0\0\0\4\215\3\3\236\201\3\3\377\241\4\4\377\266\6\6\377"
  "\202\264\6\6\377\11\263\0\0\377\322gg\377\313}}\377\306zz\377\307zz\377"
  "\311\177\177\377\260FF\377\233\0\0\377\263\6\6\377\205\262\6\6\377\2"
  "\262\7\7\377\260\3\3\377\214\256\0\0\377\3\254\0\0\377\306II\377\371"
  "\364\364\377\213\362\347\347\377\3\370\364\364\377\240NN\377\217\0\0"
  "\377\202\262\0\0\377\3\236\1\1\377\201\3\3\377\215\3\3\236\210\0\0\0"
  "\0\4\215\3\3v\201\3\3\377\224\3\3\377\262\4\4\377\202\261\5\5\377\11"
  "\260\2\2\377\272!!\377\312\177\177\377\310\200\200\377\311\201\201\377"
  "\305vv\377\226\11\11\377\233\1\1\377\262\5\5\377\203\260\5\5\377\2\257"
  "\4\4\377\255\2\2\377\216\254\0\0\377\4\253\0\0\377\256\4\4\377\371\365"
  "\365\377\362\352\352\377\211\362\347\347\377\4\363\351\351\377\371\366"
  "\366\377z\10\10\377\224\0\0\377\202\260\0\0\377\3\223\2\2\377\202\3\3"
  "\377\215\3\3v\210\0\0\0\0\21\215\3\3:\202\3\3\377\207\2\2\377\252\0\0"
  "\377\251\0\0\377\252\1\1\377\253\1\1\377\253\0\0\377\251\16\16\377\247"
  "77\377\24622\377\223\5\5\377\217\0\0\377\244\2\2\377\255\2\2\377\253"
  "\2\2\377\252\1\1\377\222\252\0\0\377\4\247\0\0\377\333\226\226\377\364"
  "\360\360\377\362\350\350\377\207\362\347\347\377\12\362\350\350\377\366"
  "\357\357\377\311\232\232\377p\0\0\377\242\0\0\377\255\0\0\377\257\0\0"
  "\377\207\2\2\377\202\3\3\377\215\3\3:\210\0\0\0\0\4\220\3\3\4\206\3\3"
  "\362\201\3\3\377\245\0\0\377\203\246\0\0\377\3\247\0\0\377\250\0\0\377"
  "\234\0\0\377\202\224\0\0\377\2\240\0\0\377\251\0\0\377\225\250\0\0\377"
  "\5\247\0\0\377\253\10\10\377\355\332\332\377\364\356\356\377\362\350"
  "\350\377\205\362\347\347\377\13\362\350\350\377\365\356\356\377\353\334"
  "\334\377}\13\13\377{\0\0\377\254\0\0\377\253\0\0\377\251\0\0\377\200"
  "\3\3\377\206\3\3\362\220\3\3\4\211\0\0\0\0\4\212\3\3\256\201\3\3\377"
  "\226\1\1\377\245\0\0\377\203\244\0\0\377\205\247\0\0\377\227\246\0\0"
  "\377\16\244\0\0\377\252\30\30\377\345\322\322\377\366\362\362\377\364"
  "\353\353\377\362\350\350\377\362\347\347\377\362\350\350\377\364\353"
  "\353\377\367\361\361\377\350\325\325\377\205\32\32\377q\0\0\377\233\0"
  "\0\377\202\251\0\0\377\3\227\1\1\377\201\3\3\377\212\3\3\256\212\0\0"
  "\0\0\4\215\3\3S\202\3\3\377\207\2\2\377\244\0\0\377\203\242\0\0\377\1"
  "\243\0\0\377\234\244\0\0\377\22\242\0\0\377\226\0\0\377\261qq\377\347"
  "\325\325\377\373\370\370\377\372\367\367\377\373\370\370\377\351\326"
  "\326\377\264ss\377v\0\0\377r\0\0\377\223\0\0\377\250\0\0\377\247\0\0"
  "\377\250\0\0\377\210\2\2\377\202\3\3\377\215\3\3S\212\0\0\0\0\5\220\3"
  "\3\7\206\3\3\355\201\3\3\377\233\0\0\377\241\0\0\377\203\240\0\0\377"
  "\236\242\0\0\377\20\230\0\0\377z\0\0\377v\6\6\377\203\32\32\377y\6\6"
  "\377m\0\0\377p\0\0\377|\0\0\377\231\0\0\377\246\0\0\377\244\0\0\377\245"
  "\0\0\377\235\0\0\377\201\3\3\377\206\3\3\355\220\3\3\7\213\0\0\0\0\4"
  "\212\3\3\203\202\3\3\377\212\2\2\377\237\0\0\377\203\236\0\0\377\1\237"
  "\0\0\377\235\240\0\0\377\11\241\0\0\377\242\0\0\377\235\0\0\377\221\0"
  "\0\377\216\0\0\377\221\0\0\377\234\0\0\377\245\0\0\377\243\0\0\377\202"
  "\242\0\0\377\4\243\0\0\377\213\2\2\377\202\3\3\377\212\3\3\203\214\0"
  "\0\0\0\5\220\3\3\24\206\3\3\365\201\3\3\377\230\0\0\377\235\0\0\377\202"
  "\234\0\0\377\1\235\0\0\377\240\236\0\0\377\202\237\0\0\377\202\241\0"
  "\0\377\203\240\0\0\377\5\241\0\0\377\233\0\0\377\201\3\3\377\206\3\3"
  "\365\220\3\3\24\215\0\0\0\0\4\212\3\3\210\202\3\3\377\207\2\2\377\233"
  "\0\0\377\203\232\0\0\377\1\233\0\0\377\241\234\0\0\377\1\235\0\0\377"
  "\205\236\0\0\377\3\206\2\2\377\202\3\3\377\212\3\3\210\216\0\0\0\0\5"
  "\220\3\3\14\206\3\3\342\202\3\3\377\217\1\1\377\231\0\0\377\202\230\0"
  "\0\377\1\231\0\0\377\241\232\0\0\377\1\233\0\0\377\203\234\0\0\377\5"
  "\235\0\0\377\221\1\1\377\202\3\3\377\206\3\3\342\220\3\3\14\217\0\0\0"
  "\0\6\215\3\3P\206\3\3\377\202\3\3\377\223\0\0\377\227\0\0\377\226\0\0"
  "\377\242\230\0\0\377\1\231\0\0\377\202\232\0\0\377\5\233\0\0\377\225"
  "\0\0\377\203\3\3\377\206\3\3\377\215\3\3P\221\0\0\0\0\6\212\3\3\212\202"
  "\3\3\377\203\3\3\377\223\0\0\377\224\0\0\377\225\0\0\377\241\226\0\0"
  "\377\1\227\0\0\377\202\230\0\0\377\4\225\0\0\377\203\3\3\377\202\3\3"
  "\377\212\3\3\212\222\0\0\0\0\6\220\3\3\2\206\3\3\266\202\3\3\377\205"
  "\2\2\377\224\0\0\377\223\0\0\377\242\224\0\0\377\6\226\0\0\377\227\0"
  "\0\377\205\2\2\377\202\3\3\377\206\3\3\266\220\3\3\2\223\0\0\0\0\6\215"
  "\3\3\16\206\3\3\326\202\3\3\377\207\2\2\377\225\2\2\377\223\2\2\377\240"
  "\222\0\0\377\6\224\1\1\377\231\3\3\377\207\2\2\377\202\3\3\377\206\3"
  "\3\326\215\3\3\16\225\0\0\0\0\10\215\3\3$\206\3\3\353\202\3\3\377\205"
  "\3\3\377\226\6\6\377\231\7\7\377\222\2\2\377\217\0\0\377\232\220\0\0"
  "\377\10\217\0\0\377\222\2\2\377\232\11\11\377\232\7\7\377\205\4\4\377"
  "\202\3\3\377\206\3\3\353\215\3\3$\227\0\0\0\0\10\215\2\2$\206\3\3\330"
  "\202\3\3\377\204\2\2\377\227\11\11\377\240\20\20\377\235\17\17\377\222"
  "\5\5\377\202\215\0\0\377\224\216\0\0\377\202\215\0\0\377\10\223\5\5\377"
  "\235\16\16\377\242\24\24\377\230\15\15\377\204\4\4\377\202\3\3\377\206"
  "\3\3\326\215\3\3$\231\0\0\0\0\13\215\2\2\16\206\3\3\275\202\3\3\377\203"
  "\2\2\377\223\14\14\377\243\31\31\377\245\34\34\377\244\35\35\377\231"
  "\20\20\377\214\1\1\377\212\0\0\377\202\213\0\0\377\214\214\0\0\377\202"
  "\213\0\0\377\7\212\0\0\377\215\0\0\377\231\20\20\377\244\36\36\377\250"
  "\40\40\377\247\34\34\377\224\16\16\377\202\202\2\2\377\2\206\3\3\271"
  "\215\3\3\16\233\0\0\0\0\14\220\3\3\2\212\2\2\222\206\3\3\377\202\2\2"
  "\377\211\7\7\377\240\33\33\377\253((\377\254++\377\25611\377\245&&\377"
  "\232\27\27\377\215\6\6\377\202\207\0\0\377\202\210\0\0\377\204\211\0"
  "\0\377\202\210\0\0\377\202\207\0\0\377\14\215\6\6\377\233\30\30\377\245"
  "''\377\25744\377\25611\377\255,,\377\242\40\40\377\210\10\10\377\201"
  "\2\2\377\206\2\2\377\212\3\3\213\220\3\3\2\236\0\0\0\0\17\215\2\2R\206"
  "\3\3\344\202\2\2\377\201\1\1\377\215\16\16\377\245))\377\261::\377\263"
  "\77\77\377\265DD\377\271JJ\377\266GG\377\25155\377\242**\377\235##\377"
  "\227\32\32\377\202\221\17\17\377\17\227\32\32\377\235##\377\244++\377"
  "\25166\377\266JJ\377\273NN\377\270JJ\377\267EE\377\265AA\377\247..\377"
  "\216\17\17\377\201\1\1\377\202\2\2\377\206\3\3\343\215\3\3Q\241\0\0\0"
  "\0\16\220\3\3\14\212\3\3\213\206\2\2\366\202\1\1\377\200\0\0\377\212"
  "\13\13\377\236((\377\266II\377\276WW\377\300\\\\\377\301aa\377\303ee"
  "\377\305ii\377\306kk\377\203\310mm\377\15\306kk\377\305ii\377\303ff\377"
  "\301bb\377\300\\\\\377\267OO\377\241++\377\212\14\14\377\200\0\0\377"
  "\202\1\1\377\206\2\2\365\212\3\3\210\220\3\3\14\244\0\0\0\0\32\220\3"
  "\3\24\212\3\3\204\206\2\2\356\202\1\1\377\201\0\0\377\177\0\0\377\211"
  "\15\15\377\233((\377\251@@\377\267XX\377\303mm\377\311yy\377\316\201"
  "\201\377\317\202\202\377\312zz\377\305pp\377\271[[\377\253CC\377\234"
  "++\377\211\16\16\377\177\0\0\377\201\0\0\377\202\1\1\377\206\2\2\355"
  "\212\3\3\204\220\3\3\24\250\0\0\0\0\11\212\0\0\7\207\0\0S\210\0\0\256"
  "\207\0\0\362\206\0\0\377\204\0\0\377\202\0\0\377\201\0\0\377\177\0\0"
  "\377\204~\0\0\377\11\177\0\0\377\201\0\0\377\202\0\0\377\204\0\0\377"
  "\206\0\0\377\207\0\0\362\210\0\0\256\207\0\0S\212\0\0\7\232\0\0\0\0\32"
  "\307\307\307\3\274\274\274\10\264\264\264\16\255\255\255\24\246\246\246"
  "\34\236\236\236$\230\230\230-\220\220\2206\212\212\212A\204\204\204L"
  "}}}Uwww[pppbjjjicccpY\\\\wPWW~FPP\205AKK\215<BB\227M**\275\\\30\30\341"
  "f\17\17\357u\11\11\366\200\4\4\373\206\3\3\375\202\213\2\2\377\32\206"
  "\3\3\375\200\4\4\374t\10\10\367f\16\16\360Z\27\27\342K''\2759\77\77\226"
  ">HH\214CMM\205MSS~UYYv___pggghlllbttt[zzzR\200\200\200G\206\206\206<"
  "\215\215\2152\225\225\225(\233\233\233\40\243\243\243\30\251\251\251"
  "\20\261\261\261\12\267\267\267\4\302\302\302\0\205\0\0\0\0_\325\325\325"
  "\33\335\335\335&\326\326\3260\316\316\3169\303\303\303C\267\267\267K"
  "\253\253\253R\245\245\245X\236\236\236_\227\227\227f\221\221\221l\212"
  "\212\212s\203\203\203z}}}\200vvv\207ooo\216iii\224bbb\233[[[\242SSS\250"
  "NNN\256GGG\265@@@\274:::\302233\311+..\317\"''\326\33!!\334\22\32\32"
  "\342\14\25\25\350\6\17\17\355\1\11\11\362\0\10\10\362\2\14\14\360\10"
  "\21\21\352\20\30\30\345\27\36\36\337\37$$\331&))\322/00\314555\306=="
  "=\277CCC\270JJJ\262PPP\254XXX\245___\236eee\230lll\221sss\212yyy\204"
  "\200\200\200}\207\207\207v\215\215\215p\224\224\224i\233\233\233b\241"
  "\241\241\\\250\250\250U\264\264\264M\277\277\277B\313\313\3139\322\322"
  "\322/\332\332\332#\321\321\321\31\333\333\333#\323\323\323*\315\315\315"
  "0\306\306\3066\300\300\300=\272\272\272C\263\263\263J\255\255\255P\247"
  "\247\247V\240\240\240\\\232\232\232b\224\224\224i\216\216\216o\210\210"
  "\210u\201\201\201|{{{\202uuu\210ooo\216iii\224ccc\231^^^\237XXX\244S"
  "SS\251NNN\257III\263EEE\270AAA\274===\300999\303777\306555\307\20255"
  "5\310>444\310666\306888\304;;;\302>>>\276CCC\272GGG\266LLL\261QQQ\254"
  "VVV\247[[[\242aaa\234fff\226lll\220rrr\213xxx\204~~~~\204\204\204x\213"
  "\213\213r\221\221\221l\227\227\227f\235\235\235`\244\244\244Y\252\252"
  "\252S\260\260\260M\267\267\267F\275\275\275@\303\303\303:\312\312\312"
  "3\320\320\320,\327\327\327&\351\351\351\23\344\344\344\31\336\336\336"
  "\37\330\330\330%\321\321\321+\313\313\3131\306\306\3066\300\300\300<"
  "\272\272\272B\264\264\264H\257\257\257M\252\252\252S\244\244\244X\237"
  "\237\237^\232\232\232b\225\225\225h\220\220\220l\213\213\213q\206\206"
  "\206v\203\203\203{~~~\177{{{\202www\206sss\212ppp\216mmm\220jjj\223h"
  "hh\226fff\227eee\230ddd\232\203ccc\232=ddd\231eee\230ggg\226iii\224j"
  "jj\222nnn\217ppp\214ttt\210xxx\204|||\201\177\177\177}\204\204\204x\212"
  "\212\212t\215\215\215o\222\222\222j\227\227\227e\234\234\234`\241\241"
  "\241[\247\247\247V\254\254\254P\262\262\262K\267\267\267E\276\276\276"
  "@\304\304\304:\312\312\3124\317\317\317.\325\325\325(\333\333\333\"\341"
  "\341\341\34\346\346\346\26\373\373\373\1\377\377\377\4\377\377\377\7"
  "\377\377\377\14\376\376\376\21\362\362\362\26\346\346\346\34\333\333"
  "\333!\327\327\327&\322\322\322*\315\315\315/\310\310\3104\304\304\304"
  "9\300\300\300=\273\273\273A\267\267\267F\263\263\263J\257\257\257N\254"
  "\254\254Q\250\250\250T\245\245\245X\242\242\242[\237\237\237^\235\235"
  "\235`\232\232\232b\230\230\230d\226\226\226f\225\225\225h\223\223\223"
  "j\222\222\222j\221\221\221k\203\221\221\221l\36\222\222\222k\223\223"
  "\223j\224\224\224i\226\226\226g\227\227\227f\231\231\231d\233\233\233"
  "b\236\236\236_\241\241\241\\\244\244\244Y\247\247\247V\252\252\252S\256"
  "\256\256O\261\261\261L\265\265\265H\271\271\271D\276\276\276\77\302\302"
  "\302;\306\306\3066\313\313\3132\320\320\320-\324\324\324(\331\331\331"
  "#\336\336\336\36\352\352\352\31\364\364\364\24\377\377\377\16\377\377"
  "\377\12\377\377\377\5\370\370\370\3\206\0\0\0\0\27\372\372\372\2\366"
  "\366\366\4\361\361\361\6\355\355\355\10\353\353\353\14\350\350\350\20"
  "\345\345\345\24\343\343\343\30\337\337\337\33\334\334\334\37\331\331"
  "\331\"\326\326\326%\323\323\323(\320\320\320+\316\316\316.\313\313\313"
  "0\311\311\3112\307\307\3074\306\306\3066\304\304\3048\303\303\3039\301"
  "\301\301:\300\300\300<\202\277\277\277<\203\277\277\277=\31\277\277\277"
  "<\300\300\300<\301\301\301;\302\302\302:\303\303\3038\305\305\3057\306"
  "\306\3065\310\310\3104\312\312\3122\315\315\315/\317\317\317,\322\322"
  "\322*\325\325\325&\327\327\327$\332\332\332\40\336\336\336\35\341\341"
  "\341\31\344\344\344\26\347\347\347\22\352\352\352\16\354\354\354\12\357"
  "\357\357\10\363\363\363\5\370\370\370\3\373\373\373\1\205\0\0\0\0"};


#endif
